/****************************************************************************
* Rock Paper Scissors for Nintendo Wii
* Version: 0.2
*
* Author: Forgotten Linkz
* Created: 20/06/2008
*
* Special Thanks to:
*  - teknecal:  i used most of the code from his game matching_cards
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <jpeg/jpgogc.h>
#include <wiiuse/wpad.h>

extern char rock_picdata[];
extern int  rock_piclength;
extern char paper_picdata[];
extern int  paper_piclength;
extern char scissor_picdata[];
extern int  scissor_piclength;
extern char questionmark_picdata[];
extern int  questionmark_piclength;
extern char player1_picdata[];
extern int  player1_piclength;
extern char player2_picdata[];
extern int  player2_piclength;
extern char ready_picdata[];
extern int  ready_piclength;
extern char single_picdata[];
extern int  single_piclength;
extern char single_selected_picdata[];
extern int  single_selected_piclength;
extern char two_picdata[];
extern int  two_piclength;
extern char two_selected_picdata[];
extern int  two_selected_piclength;
extern char logo_picdata[];
extern int  logo_piclength;

JPEGIMG rock_image, paper_image, scissor_image, 
		questionmark_image, player1_image, player2_image,
		ready_image, single_image, single_selected_image,
		two_image, two_selected_image, logo_image;

// 2D Video Globals
static u32 *xfb[2] = { NULL, NULL };
static GXRModeObj *vmode;

bool selectedP1 = false;
bool selectedP2 = false;
bool playing = false;

int p1_score = 0;
int p2_score = 0;
int p1_image_x = 200;
int p1_image_y = 60;
int p2_image_x = 200;
int p2_image_y = 320;

// Initialise video and controls
void Initialise() {
	
	// Initialise the video system
	VIDEO_Init();
	
	// This function initialises the attached controllers
	WPAD_Init();
	
	// Obtain the preferred video mode from the system
	// This will correspond to the settings in the Wii menu
	vmode = VIDEO_GetPreferredMode(NULL);
	
	// Allocate memory for the display in the uncached region
	xfb[0] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(vmode));
	xfb[1] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(vmode));
	
	// Initialise the console, required for printf
	console_init(xfb[0],20,20,vmode->fbWidth,vmode->xfbHeight,vmode->fbWidth*VI_DISPLAY_PIX_SZ);
	
	// Set up the video registers with the chosen mode
	VIDEO_Configure(vmode);
	
	// Tell the video hardware where our display memory is
	VIDEO_SetNextFramebuffer(xfb[0]);
	
	// Make the display visible
	VIDEO_SetBlack(FALSE);

	// Flush the video register changes to the hardware
	VIDEO_Flush();

	// Wait for Video setup to complete
	VIDEO_WaitVSync();
	if(vmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();
}

// Quick Vsync refresh function
void SetScreen() {
	VIDEO_SetNextFramebuffer(xfb[0]);
	VIDEO_Flush();
	VIDEO_WaitVSync();
}

// Clears the screen. Sets screen to black.
void clearscreen() {
	int x,y;
	for(y=0;y<506;y++)
		for(x=0;x<320;x++)
			xfb[0][x+320*y]=COLOR_BLACK;
}

// Display a JPEG image
void display_jpeg(JPEGIMG jpeg, int offsetx, int offsety) {

	unsigned int *jpegout = (unsigned int *) jpeg.outbuffer;
	
	int i,j;
	for(i=0;i<=(jpeg.width-(jpeg.width/2)-2);i++)
		for(j=1;j<=jpeg.height-2;j++) 
			xfb[0][(i+offsetx)+320*(j+16+offsety)]=jpegout[i+(jpeg.width-(jpeg.width/2))*j];
}

void resetgame() {
	display_jpeg(player1_image, p1_image_x, p1_image_y-50);
	display_jpeg(questionmark_image, p1_image_x, p1_image_y);
	
	display_jpeg(player2_image, p2_image_x, p2_image_y-50);
	display_jpeg(questionmark_image, p2_image_x, p2_image_y);
	
	selectedP1 = false;
	selectedP2 = false;
}

void keep_score(int score) {

	if( score == 1 ) p1_score++;
	if( score == -1 ) p2_score++;
}

void reset_score() {

	p1_score = 0;
	p2_score = 0;
}

int who_won(int player1, int player2) {
	int result;
	if(player1 == 1 && player2 == 1) result = 0;
	if(player1 == 2 && player2 == 2) result = 0;
	if(player1 == 3 && player2 == 3) result = 0;
	
	if(player1 == 1 && player2 == 2) result = -1;
	if(player1 == 1 && player2 == 3) result = 1;
	
	if(player1 == 2 && player2 == 1) result = 1;
	if(player1 == 2 && player2 == 3) result = -1;
	
	if(player1 == 3 && player2 == 1) result = -1;
	if(player1 == 3 && player2 == 2) result = 1;
	
	keep_score(result);
	return result;
}

void show_hands(int p1, int p2) {

	switch(p1) {
		case 1: 	display_jpeg(rock_image, p1_image_x, p1_image_y);
					break;
		case 2:		display_jpeg(paper_image, p1_image_x, p1_image_y);
					break;
		case 3:		display_jpeg(scissor_image, p1_image_x, p1_image_y);
					break;
	}

	switch(p2) {
		case 1: 	display_jpeg(rock_image, p2_image_x, p2_image_y);
					break;
		case 2:		display_jpeg(paper_image, p2_image_x, p2_image_y);
					break;
		case 3:		display_jpeg(scissor_image, p2_image_x, p2_image_y);
					break;
	}
}

void show_winner(int winner) {
	
	bool done = false;
	bool printed = false;
	while(!done){
		WPAD_ScanPads();
		u32 pressedP1 = WPAD_ButtonsDown(0);
		u32 pressedP2 = WPAD_ButtonsDown(1);
		
		if(!printed){
			if(winner == 1) printf("Player 1 wins! \n\n");
			if(winner == 0) printf("It's a tie! \n\n");
			if(winner == -1) printf("Player 2 wins! \n\n");
			printf("Press A to play again \nOr B to get back to the menu");
			printed = true;
		}
		
		if ( pressedP1 & WPAD_BUTTON_A ) {
			done = true;
		}
		if ( pressedP1 & WPAD_BUTTON_B ) {
			playing = false;
			done = true;
		}
		if ( pressedP1 & WPAD_BUTTON_HOME ) {
			exit(0);
		}
			
		if ( pressedP2 & WPAD_BUTTON_A ) {
			done = true;
		}
		if ( pressedP2 & WPAD_BUTTON_B ) {
			playing = false;
			done = true;
		}
		if ( pressedP2 & WPAD_BUTTON_HOME ) {
			exit(0);
		}
	}
	
	SetScreen();
}

// Selection of hands (single or two player)
void select_hand(bool single) {
	
	playing = true;
	
	while(playing) {
		clearscreen();
		resetgame();
		int selectionP1 = 0;
		int selectionP2 = 0;
		bool done = false;
		
		printf("\x1b[2;5H");
		printf("Select your hand:");
		
		printf("\n\n");
		printf("up for rock");
		
		printf("\n\n");
		printf("right for paper");
		
		printf("\n\n");
		printf("down for scissors");
		
		printf("\n\n");
		printf("Press the A button when you're ready!");
		
		printf("\n\n");
		printf("========================================");
		printf("\n\n");
		printf("The score:\n");
		printf("Player 1: %d points, Player 2: %d points", p1_score, p2_score);
		printf("\n\n");
		printf("========================================");
		printf("\n\n");
		
		// Waiting for the chooices to be made by the players
		while(!done) {
			WPAD_ScanPads();
		
			u32 pressedP1 = WPAD_ButtonsDown(0);
			
			// Selection Player 1
			if ( (pressedP1 & WPAD_BUTTON_UP) && !selectedP1 ) { 
				selectionP1 = 1; 
				WPAD_Rumble(0, 1); 
				usleep(200000); 
				WPAD_Rumble(0, 0);
			} 
			if ( (pressedP1 & WPAD_BUTTON_RIGHT) && !selectedP1 ) {
				selectionP1 = 2;
				WPAD_Rumble(0, 1); 
				usleep(200000); 
				WPAD_Rumble(0, 0);
			}
			if ( (pressedP1 & WPAD_BUTTON_DOWN) && !selectedP1 ) {
				selectionP1 = 3;
				WPAD_Rumble(0, 1); 
				usleep(200000); 
				WPAD_Rumble(0, 0);
			}
			if ( (pressedP1 & WPAD_BUTTON_A) && selectionP1 > 0 ) { 
				selectedP1 = true;
				display_jpeg(ready_image, p1_image_x, p1_image_y);
			}
			if ( pressedP1 & WPAD_BUTTON_HOME ) exit(0);
			
			// Only use P2 controls if it's a 2 player mode
			if(!single) {
				
				u32 pressedP2 = WPAD_ButtonsDown(1);
				// Selection Player 2
				if ( (pressedP2 & WPAD_BUTTON_UP) && !selectedP2 ) { 
					selectionP2 = 1; 
					WPAD_Rumble(1, 1); 
					usleep(200000); 
					WPAD_Rumble(1, 0);
				} 
				if ( (pressedP2 & WPAD_BUTTON_RIGHT) && !selectedP2 ) {
					selectionP2 = 2;
					WPAD_Rumble(1, 1); 
					usleep(200000); 
					WPAD_Rumble(1, 0);
				}
				if ( (pressedP2 & WPAD_BUTTON_DOWN) && !selectedP2 ) {
					selectionP2 = 3;
					WPAD_Rumble(1, 1); 
					usleep(200000); 
					WPAD_Rumble(1, 0);
				}
				if ( (pressedP2 & WPAD_BUTTON_A) && selectionP2 > 0 ) {
					selectedP2 = true;
					display_jpeg(ready_image, p2_image_x, p2_image_y);
				}
				if ( pressedP2 & WPAD_BUTTON_HOME ) exit(0);
			} else {
				// Randomly select opponent
				selectionP2 = rand() % 3 + 1;
			}
			
			if((selectedP1 && selectedP2) || (selectedP1 && single)) done = true;
		}
		
		show_hands(selectionP1, selectionP2);
		show_winner(who_won(selectionP1, selectionP2));
	}
}	

// Single or Two player mode selection
void menu() {
	
	bool done = false;
	bool single = true;
	
	// Position of images
	int x = 90;
	int y = 200;
	
	display_jpeg(logo_image, x-20, y-180);
	
	display_jpeg(single_selected_image, x, y);
	display_jpeg(two_image, x, y+100 );
	
	while(!done) {
		WPAD_ScanPads();
	
		u32 pressed = WPAD_ButtonsDown(0);
		
		if( (pressed & WPAD_BUTTON_DOWN) && single ) {
			display_jpeg(two_selected_image, x, y+100);
			display_jpeg(single_image, x, y);
			single = false;
		}
		if( (pressed & WPAD_BUTTON_UP) && !single ) {
			display_jpeg(two_image, x, y+100);
			display_jpeg(single_selected_image, x, y);
			single = true;
		}
		if ( pressed & WPAD_BUTTON_A) {
			done = true;
		}
		if ( pressed & WPAD_BUTTON_HOME ) exit(0);
	}
	
	clearscreen();
	resetgame();
	reset_score();
	select_hand(single);
}

void game() {

	while(1) {
		clearscreen();
		menu();
		resetgame();
		SetScreen();
	}
}

int main () {

	Initialise();
	
	// Decompress the Jpegs	
	memset(&rock_image, 0, sizeof(JPEGIMG));
	rock_image.inbuffer = rock_picdata;
	rock_image.inbufferlength = rock_piclength;
	JPEG_Decompress(&rock_image);
	
	memset(&paper_image, 0, sizeof(JPEGIMG));
	paper_image.inbuffer = paper_picdata;
	paper_image.inbufferlength = paper_piclength;
	JPEG_Decompress(&paper_image);
	
	memset(&scissor_image, 0, sizeof(JPEGIMG));
	scissor_image.inbuffer = scissor_picdata;
	scissor_image.inbufferlength = scissor_piclength;
	JPEG_Decompress(&scissor_image);

	memset(&questionmark_image, 0, sizeof(JPEGIMG));
	questionmark_image.inbuffer = questionmark_picdata;
	questionmark_image.inbufferlength = questionmark_piclength;
	JPEG_Decompress(&questionmark_image);
	
	memset(&player1_image, 0, sizeof(JPEGIMG));
	player1_image.inbuffer = player1_picdata;
	player1_image.inbufferlength = player1_piclength;
	JPEG_Decompress(&player1_image);

	memset(&player2_image, 0, sizeof(JPEGIMG));
	player2_image.inbuffer = player2_picdata;
	player2_image.inbufferlength = player2_piclength;
	JPEG_Decompress(&player2_image);
	
	memset(&ready_image, 0, sizeof(JPEGIMG));
	ready_image.inbuffer = ready_picdata;
	ready_image.inbufferlength = ready_piclength;
	JPEG_Decompress(&ready_image);

	memset(&single_image, 0, sizeof(JPEGIMG));
	single_image.inbuffer = single_picdata;
	single_image.inbufferlength = single_piclength;
	JPEG_Decompress(&single_image);
	
	memset(&single_selected_image, 0, sizeof(JPEGIMG));
	single_selected_image.inbuffer = single_selected_picdata;
	single_selected_image.inbufferlength = single_selected_piclength;
	JPEG_Decompress(&single_selected_image);

	memset(&two_image, 0, sizeof(JPEGIMG));
	two_image.inbuffer = two_picdata;
	two_image.inbufferlength = two_piclength;
	JPEG_Decompress(&two_image);
	
	memset(&two_selected_image, 0, sizeof(JPEGIMG));
	two_selected_image.inbuffer = two_selected_picdata;
	two_selected_image.inbufferlength = two_selected_piclength;
	JPEG_Decompress(&two_selected_image);
	
	memset(&logo_image, 0, sizeof(JPEGIMG));
	logo_image.inbuffer = logo_picdata;
	logo_image.inbufferlength = logo_piclength;
	JPEG_Decompress(&logo_image);
	
	game();
	
	return 0;
}
